/*###########################################################################
#                                                                           #
#    		               wb_t@C								#
#                                                                           #
###########################################################################*/
#ifndef __PROJECT_H__
#define __PROJECT_H__

#define VERSION "5.24"	/* t@[o[W	*/

/* et@CCN[h			*/
#include <stdio.h>
#include <string.h>

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <util/crc16.h>

#include "FatFs/tff.h"
#include "FatFs/diskio.h"
#include "ar1000.h"
#include "at45db161d.h"
#include "string_func.h"
#include "vs1053b.h"
#include "zy_fgd1442701v1.h"
#include "flash_sys.h"
#include "graphic.h"
#include "crc.h"
#include "timer.h"
#include "keys.h"
#include "console.h"
#include "unicode.h"

#define ON 1
#define OFF 0

#define SCALE_CONVERT(val,rg1,rg2) ( ( ( ( ( (uint32_t)(val) * (uint32_t)(rg2) ) * 10L ) + 5L ) / (uint32_t)(rg1) ) / 10L )

#define POWER_BUTTON() ( PINF & _BV(5) )			/* d{^ Hi:OFF Low:ON	*/
#define POWER_HOLD() ( PORTF |= _BV(4) )			/* dOnz[h			*/
#define POWER_OFF() ( PORTF &= ~_BV(4) )			/* dOnz[h		*/

#define IN_CHG() ( PINF & _BV(6) )					/* Hi:d Low:[d		*/

#define SD_INS() ( PIND & _BV(6) )					/* Hi:} Low:}		*/

#define VOLTAGE_R7 150				/* BATT(+)̒Rl(K)					*/
#define VOLTAGE_R8 47				/* GND̒Rl(K)						*/
#define REF_VOLTAGE 110				/* t@Xd 1.1V=110 or 2.56V=256	*/
#define AD_CH 7						/* A/Dϊ`lԍ						*/

/* tbVt@CCfbNXԍ			*/
#define FL_UNI2SJIS 0				/* UNI2SJIS.BIN : UNICODE -> SJISϊ		*/
#define FL_SJIS2UNI 1				/* SJIS2UNI.BIN : SJISE -> UNICODEϊ		*/
#define FL_GOTHN12X 2				/* GOTHN12X.TLF : 12x12ptHg			*/
#define FL_GOTZN12X 3				/* GOTZN12X.TLF : 12x12SptHg			*/
#define FL_MGOHN16X 4				/* MGOHN16X.TLF : 16x16ptHg			*/
#define FL_MGOZN16X 5				/* MGOZN16X.TLF : 16x16SptHg			*/
#define FL_VS_IMAFIX 10				/* VS1053B : 								*/
#define FL_VS_VORBISFIX 11			/* VS1053B : 								*/
#define FL_VS_SPECTRUM 12			/* VS1053B : 								*/
#define FL_VS_OGG   15				/* VS1053B : OGGGR[_[擪CfbNX*/
#define FL_ICON_START_INDEX 30		/* ICONt@C擪CfbNX				*/
#define FL_BG_START_INDEX 200		/* wi摜LbV擪CfbNX		*/
#define FL_BG_SYSTEM_INDEX 255		/* VXewi摜CfbNX				*/

/* FCfbNX						*/
#define BLACK RGB(0,0,0)
#define RED RGB(255,0,0)
#define GREEN RGB(0,255,0)
#define LITEBLUE (0,255,255)
#define BLUE RGB(0,0,255)
#define YELLOW RGB(255,255,0)
#define PURPLE RGB(255,0,255)
#define WHITE RGB(255,255,255)

/* Xe[VXg萔				*/
#define FM_FILE_NAME "FM_LIST.TXT"				/* Xe[VXgt@C					*/
#define FM_LIST_MAX	32							/* ǗXe[V						*/
#define FM_STATION_NAME_LENGTH 32				/* Xe[V̍ő咷						*/
#define FM_STATION_LIST_SIZE ( 6 + ( sizeof(FM_ITEM) * FM_LIST_MAX ) )
#define FM_STATION_ITEM_ADR(idx) ( EEP_STATION_LIST_ADDRESS + 8 + ( sizeof(FM_ITEM) * idx ) )

/* EEPAhX`						*/
#define EEP_SYSTEM_ADDRESS 0x0000				/* VXeϐۑ擪AhX					*/
#define EEP_STATION_LIST_ADDRESS 0x0400			/* Xe[VXgۑ擪AhX			*/

#define HIS_SHUTDOWN_COUNT 			  5			/* Vbg_Eێ						*/

#define SEEK_VALUE_MAX				 87			/* V[No[ől								*/

#define SP_INTERVAL					  8			/* XyAi\Ԋu 10msecP					*/
#define RSSI_INTERVAL  				 15			/* RSSIOt\Ԋu(10msecP)				*/

#define START_SCROLL_TIME 			200			/* XN[Jn(10msecP)				*/
#define START_SCROLL_TIME2 			 50			/* XN[Jn(10msecP) screen011/02	*/
#define SCROLL_INTERVAL				  7			/* XN[Ԋu(10msecP)					*/

#define AD_INTERVAL_TIME			200			/* drd擾Ԋu(10msecP)					*/

#define SHUTDOWN_LOW_BATT			325			/* ̓d̏ꍇ̓Vbg_E			*/
#define NOT_PROCESS_SHUTDOWN_TIME	600			/* pԃ^CAEg(100msec)			*/

#define VOLUME_MAXVALUE 			 12			/* ʍől( 0 ` VOLUME_MAXVALUE )ύXs	*/

#define PLAY_LIST_ITEM_MAX			100			/* XgACeő吔 (MAX 254)				*/
#define PLAY_LIST_PATH_LEGNTH		130			/* fBNgtpXő咷()				*/

#define FADE_INTERVAL_TIME		 	 15			/* ʃtF[hCEAEgԊu(10msecP)		*/
#define FADE_IN_STEP				  8			/* ʃtF[hCXebv					*/
#define FADE_OUT_STEP				  4			/* ʃtF[hAEgXebv					*/

/* Vbg_E萔				*/
typedef enum {
	 SHUTDOWN_NONE			= 0
	,SHUTDOWN_USER			= 1					/* [U[ɂVbg_E					*/
	,SHUTDOWN_TIMER			= 2					/* ^C}[ɂVbg_E					*/
	,SHUTDOWN_LOWBATT		= 3					/* drdቺɂVbg_E				*/
	,SHUTDOWN_NOT_PROCESS_TIMER = 4				/* DSP]Ԓ߂ɂVbg_E		*/
} SHUTDOWN_TYPE;

/* ʃtF[hCEAEgԒ萔	*/
/* ʃrbgZbgŎs			*/
typedef enum {
	 VOLUME_FADE_NONE 		= 0x00				/* ʃtF[hI								*/
	,VOLUME_FADE_IN	  		= 0x01				/* tF[hCJn								*/
	,VOLUME_FADE_OUT		= 0x02				/* tF[hAEgJn							*/
	,VOLUME_FADE_NEXT 		= 0xff
} VOLUME_FADE_MODE;

/* s[gĐ[h				*/
typedef enum {
	 REPET_NONE				= 0					/* ʏĐ										*/
	,REPET_FILE				= 1					/* ̃t@Cs[g						*/
	,REPET_DIRECTORY		= 2					/* fBNgs[g						*/
												/* fBNgɂTufBNg			*/
												/* ɂ͈ړȂ								*/
} REPET_TYPE;

/* v[[{^ACR			*/
typedef enum {
	 BUTTON_ICON_NONE	    = 0x00
	,BUTTON_ICON_REV		= 0x01
	,BUTTON_ICON_PLAY		= 0x02
	,BUTTON_ICON_PAUSE		= 0x04
	,BUTTON_ICON_STOP		= 0x08
	,BUTTON_ICON_FF			= 0x10
} BUTTON_ICON;

/* v[[Xe[^X				*/
/* 7bit    : G[tO			*/
/* 6bit    : DSP]tO		*/
/* 5bit    : t@CI[vtO	*/
/* 0`3bit : Xe[^Xԍ			*/
#define PLAY_STS_ERROR_BIT		0x80
#define PLAY_STS_DSP_BIT		0x40
#define PLAY_STS_OPEN_BIT		0x20
#define PLAY_STS_DSP_ERROR_BIT	0x10
typedef enum {
	 PLAY_STS_NONE          = 0x00				/* player_process()/player_play()֐̐pߒl */
	,PLAY_STS_STOP			= 0x01				/* Đ										*/
	,PLAY_STS_PLAYING		= 0x62				/* Đ										*/
	,PLAY_STS_FAINARAIZ		= 0x63				/* t@CiCY								*/
	,PLAY_STS_FINISH		= 0x04				/* RɍĐI							*/
	,PLAY_STS_PAUSE			= 0x25				/* |[Y										*/
	,PLAY_STS_ERROR			= 0x96				/* DSP]荞ݓŃG[				*/
	,PLAY_STS_NOT_INSERT	= 0x87				/* SDJ[h}								*/
	,PLAY_STS_ACCESS_ERROR  = 0x88				/* SDJ[hANZXG[						*/
	,PLAY_STS_NOT_SYPPORT   = 0x09				/* T|[gt@C							*/
} PLAYER_STS;

/* v[[R}h			*/
typedef enum {
	 PLAY_CURRENT			= 0x01				/* JgACeĐ(W[Lj		*/
	,PLAY_NEXT				= 0x02				/* ̋ȂĐ									*/
	,PLAY_BACK				= 0x03				/* ŐȂĐ								*/
} PLAY_MODE;

/* TEh`					*/
typedef enum {
	 SOUND_LOWBATT = 0							/* obe[dቺVbg_E			*/
	,SOUND_ALERT								/* obe[dቺx						*/
	,SOUND_ERROR								/* VXeG[̉							*/
	,SOUND_POWER_ON								/* dON̉									*/
	,SOUND_POWER_OFF							/* dOFF̉								*/
	,SOUND_NEXT_MUSIC							/* ̋ȍĐ̉								*/
	,SOUND_BACK_MUSIC							/* ŐȍĐ̉								*/
	,SOUND_PAUSE								/* ꎞ~										*/
	,SOUND_TOP_OF_MUSIC							/* Đ̋Ȃ̐擪ɖ߂鎞̉					*/
	,SOUND_CHANGE_MODE							/* v[[EWI[hؑւ̉				*/
	,SOUND_CHANGE_STATION						/* FMWIǕύX̉						*/
	,SOUND_SEEK_START							/* FMWIXLJn̉				*/
} SOUND_TYPE;

/* Đt@C^Cv`				*/
#define GET_DSP_STATUS_BIT 0x80
#define DO_NOT_JUMP_BIT 0x40
typedef enum {
	 F_OTHER = 0x00											/* T|[gf[^					*/
	,F_MP3 	 = GET_DSP_STATUS_BIT |                   0x01	/* MP3								*/
	,F_WMA 	 = GET_DSP_STATUS_BIT | DO_NOT_JUMP_BIT | 0x02	/* WMA								*/
	,F_AAC	 = GET_DSP_STATUS_BIT | DO_NOT_JUMP_BIT | 0x03	/* AAC								*/
	,F_OGG	 = GET_DSP_STATUS_BIT | DO_NOT_JUMP_BIT | 0x04	/* OGG								*/
	,F_MIDI	 = GET_DSP_STATUS_BIT |                   0x05	/* MIDItH[}bg0				*/
	,F_PCM	 =                      DO_NOT_JUMP_BIT | 0x06	/* jAPCM						*/
	,F_ADPCM = GET_DSP_STATUS_BIT | DO_NOT_JUMP_BIT | 0x07	/* IMA ADPCM						*/
	,F_FMRADIO = 0x08										/* FM-RADIO							*/
	,F_WAVE  = 0x1f											/* WAVE(̒lł͍ĐȂ)		*/
} PLAY_FILE_TYPE;

/* W[̍Đ[h`	*/
typedef enum {
	 RESUME_NONE = 0							/* W[񖳌							*/
	,RESUME_TOP = 1								/* t@C̐擪Đ						*/
	,RESUME_SEEK = 2							/* t@C̓rĐ						*/
	,RESUME_NEXT = 3							/* ̋ȂĐ									*/
} RESUME_MODE;

/* W[\				*/
typedef struct RESUME_INFO_t {
	RESUME_MODE mode;							/* W[̍Đ[h						*/
												/* 0:t@C̐擪Đ						*/
												/* 1:t@C̓rĐ						*/
												/* 2:̃t@CĐ							*/
	uint32_t fpos;								/* Đ𒆒ft@Cʒu					*/
	uint32_t time;								/* Đ𒆒fĐ(10msecP)			*/
	char fname[PLAY_LIST_PATH_LEGNTH + 16];		/* Đt@C								*/
} RESUME_INFO;

/* Đt@C\			*/
typedef struct PLAY_FILE_INFO_t {
	PLAY_FILE_TYPE type;						/* t@C									*/
	uint8_t bitsPerSamples;						/* Tvӂ̃rbg						*/
	uint32_t startPos;							/* f[^̊Jnʒu								*/
	uint16_t samplesPerSeconds;					/* TvOg(Hz)						*/
	uint16_t blockSize;							/* 1ubÑoCg(ACg)			*/
	uint16_t bitRate;							/* rbg[g(Kbps)							*/
	uint16_t titleBufferSize;					/* Ȗobt@̃TCY : 0 = 		*/
	uint16_t authorBufferSize;					/* ҕobt@̃TCY : 0 = 		*/
	char *title;								/* Ȗobt@ւ̃|C^ : NULL = 	*/
	char *author;								/* ҕobt@ւ̃|C^ : NULL = 	*/
} PLAY_FILE_INFO;

/* vCACe\(14bytes)	*/
typedef struct PLAY_LISTITEM_t {
	char atb;									/*  '0':t@C / '1':fBNg			*/
	char fname[13];								/* 8.3`t@C							*/
} PLAY_LISTITEM;

/* vCXg\(1534bytes)	*/
typedef struct PLAYLIST_t {
	uint8_t count;								/* LACe								*/
	uint8_t index;								/* JgACeԍ(0xff:)				*/
	char path[ PLAY_LIST_PATH_LEGNTH + 16 ];	/* JgfBNg						*/
	PLAY_LISTITEM items[ PLAY_LIST_ITEM_MAX ];	/* ACeXg								*/
} PLAYLIST;

/* Xe[V\̒`		*/
typedef struct FM_ITEM_t {
	uint16_t freq;								/* g(10{) 83.1MHz = 831 					*/
	char name[ FM_STATION_NAME_LENGTH + 4 ];	/* Xe[V								*/
} FM_ITEM;

/* Ao摜\̒`		*/
typedef struct ALBUM_INFI_t {
	char fullName[ PLAY_LIST_PATH_LEGNTH + 16 ];
	uint16_t fdate;
	uint16_t ftime;
	uint32_t size;
} ALBUM_INFO;

/* VXeϐ\̒`			*/
typedef struct SYSTEM_t {
	FONT_SET fnt1;								/* 12x12tHgZbg							*/
	FONT_SET fnt2;								/* 16x16tHgZbg							*/

	FATFS fs;									/* FatFs[N									*/
	VOLUME_FADE_MODE volumeFadeMode;			/* ʃtF[hCEAEg					*/
	FIL playFL;									/* Đt@C\ / ^Ɏgp			*/

	uint8_t screenBrightness;					/* ݂̉ʋPx								*/

	uint8_t flag_sdChange;						/* SDJ[h}ԕωtO					*/
	uint8_t flag_sdInsert;						/* SDJ[h}ԃtO						*/

	volatile uint32_t playTime;					/* ĐԃJE^(b)							*/
	volatile uint8_t tickPlayTime;				/* ĐԃJE^XVL						*/

	volatile uint8_t countDownTimer1;			/* JE^_E^C}[P(10msec)PlayTime		*/
	volatile uint8_t countDownTimer2;			/* JE^_E^C}[Q(10msec)PlayTime		*/
	volatile uint8_t countDownTimer3;			/* JE^_E^C}[R(10msec)PlayTime		*/
	volatile uint8_t countDownTimer4;			/* JE^_E^C}[S(10msec)PlayTime		*/

	volatile uint8_t flag_adTranslationCompletion;
	volatile uint8_t ADcountDownTimer;			/* A/DϊJEg_E^C}[				*/
	volatile uint16_t batteryAD;				/* drd A/Dϊl							*/

	volatile uint16_t systemTime;				/* VXe(b)								*/

	volatile uint16_t screenOffCountDownTimer;	 /* OFF^C}[(100msec) 0xffff:		*/
	volatile uint8_t tickScreenFade;			 /* ʃtF[hAEg^C~O				*/

	volatile uint16_t shutdownCountDownTimer;	/* Vbg_E^C}[(1msec)				*/
	volatile uint8_t tickShutdownCountDownTimer;/* Vbg_E^C}[lύXLtO		*/

	volatile uint16_t notProccesCountDownTimer; /* 쒆ԑ^C}[(100msec)			*/

	volatile PLAYER_STS playMode;				/* v[[̃Xe[^X						*/
	volatile uint16_t dsp_hDat0;				/* VS1053B HDAT0l								*/
	volatile uint16_t dsp_hDat1;				/* VS1053B HDAT1l								*/
	volatile uint16_t dsp_auData;				/* VS1053B AUDATAl								*/
	volatile uint16_t dsp_spectrum[ VS_SPECTRUM_VALUES ];	/* XyAif[^					*/
	volatile uint8_t enableSpectrum;			/* XyNgf[^荞݊					*/
	volatile uint8_t dspTransFlag;				/* dsp]tO								*/

	uint8_t bgIndex;							/* wi摜CfbNXԍ						*/
	uint8_t erroring;							/* G[tO(player)					*/
	uint8_t backupErroring;						/* erroringtOobNAbv					*/
	G_DRAWMODE gDrawMode;						/* `p[^								*/
	PLAYLIST playList;							/* vCXg									*/
	PLAY_FILE_INFO playInfo;					/* Đt@C							*/
	char backupPath[ PLAY_LIST_PATH_LEGNTH + 16 ];	/* Đިڸؕωop					*/
	char textBuffer[ LFN_LENGTH * 2 + 16 ];		/* ebveLXgobt@						*/

} SYSTEM;

/* EEP֕ۑp[^		*/
typedef struct EEP_t {
	uint8_t screenIndex;						/* ݕ\Ă郁j[ʔԍ			*/
	uint8_t volume;								/* (0`VOLUME_MAXVALUE)						*/
	uint8_t screenBrightness;					/* ʋPx										*/
	uint8_t systemUpdate;						/* ÑVXet@CmF 0: 1:L	*/
	uint8_t playTimeDisable;					/* sys.playTimetO 0:L !0:		*/
	uint8_t bgMode;								/* 0:ĝ 1:ftHgwi 2:wwi 9:ftHgwi+g 10:wwi+g */
	REPET_TYPE repetMode;						/* s[gĐ[h							*/
	uint8_t earSpeaker;							/* EarSpeakerx(0`3)						*/
	uint8_t hi_freq;							/* g(1`15)KHzȏ		*/
	int8_t hi_lv;								/* x1.5db݂Ŏw(-8`7)			*/
	uint8_t low_freq;							/* g(2`15) Hzȉ̎g(20Hz`150Hz) */
	uint8_t low_lv;								/* x1db݂Ŏw(0`15)			*/
	uint16_t fmFreq;							/* M̎g								*/
	uint16_t screenOffTimer;					/* OFF^C}[(100msec) 0xffff:			*/
	uint16_t shutdownTimer;						/* Vbg_E^C}[(1sec) 0xffff: 	*/
	uint32_t voltageCalibration;				/* drdA/Dϊ␳萔						*/
	uint16_t his_powerOnTime[HIS_SHUTDOWN_COUNT];/* Vbg_E܂ł̋Nԗ			*/
	uint8_t his_shutDownType[HIS_SHUTDOWN_COUNT];/* Vbg_Ẽ^Cv					*/
	RESUME_INFO resume;							/* W[								*/
	uint8_t updateAlbumIndex;					/* ɏAo摜obt@ԍ		*/
} EEP;

/* VXeϐ錾					*/
#ifdef __MAIN_C__
	SYSTEM sys;									/* dɃNAϐ		*/
	EEP eep;									/* dOFFłȂϐ		*/
#else
	extern SYSTEM sys;
	extern EEP eep;
#endif

/*									*/
/* ֐vg^Cv錾				*/
/*									*/

/* system.c */
void systemProcess( void );
void powerON( void );
void shutdown( SHUTDOWN_TYPE type );
uint16_t getBattVoltage( void );
uint16_t _getBattVoltage( void );
void loadSystemFile( uint8_t mode ,uint16_t index ,char *fname ,uint8_t ftype ,uint16_t n );
void loadSystemDefault( void );
void clearAllEEP( void );

/* play_file_info.c			*/
PLAY_FILE_TYPE getPlayFileType( char *fileName );
uint8_t getPlayFileInfo( char *fileName ,PLAY_FILE_INFO *info );
void clearFileInfo( PLAY_FILE_INFO *info );

/* play_file_list.c			*/
uint8_t playList_getItemName( PLAYLIST *list ,uint8_t index ,char *fname );
uint8_t playList_fileItem( PLAYLIST *list ,char *fname );
uint8_t playList_findFastItem( PLAYLIST *list );
uint8_t playList_moveNext( PLAYLIST *list ,REPET_TYPE repet );
uint8_t playList_moveBack( PLAYLIST *list ,REPET_TYPE repet );
uint8_t playLIst_moveParents( PLAYLIST *list );
uint8_t playList_moveChild( PLAYLIST *list );
uint8_t playList_loadDir( PLAYLIST *list ,char *pathName );

/* player.c					*/
void dspEnable( void );
void dspDisable( void );
PLAYER_STS player_process( void );
uint8_t player_afterPorcess( void );
void player_backProcess_for_submenu( void (*func)( void ) );
VOLUME_FADE_MODE player_volumeFade( VOLUME_FADE_MODE mode );
PLAYER_STS player_play( PLAY_MODE mode );
void player_pause( void );
void player_stop( void );
void player_volume( uint8_t vol );
void player_sound( SOUND_TYPE type );

/* fm_items.c				*/
uint8_t getStationItemCount( void );
uint8_t getStationItem( uint8_t itemIndex ,FM_ITEM *st );
uint8_t findStationItem( uint16_t freq ,FM_ITEM *st );
void clearStationList( void );
uint8_t compStationListCRC( void );
uint8_t compStationListTimeStamp( void );
uint8_t loadStationList( uint8_t mode ,uint16_t *line );

/* sys_draw.c		*/
uint16_t sysDraw_battery( void );
void sysDraw_scrollFirst( char *str );
void sysDraw_scrollNext( void );
void sysDraw_volume( void );
void sysDraw_spectrum( uint8_t typ );
void sysDraw_fileType( PLAY_FILE_TYPE type );
void sysDraw_bitRate( uint16_t bitRate );
void sysDraw_sampleRate( uint16_t sampleRate );
void sysDraw_playTime( uint16_t time );
void sysDraw_playButton( BUTTON_ICON type ,uint8_t sw );
void sysDraw_repetButton( void );
void sysDraw_shutDownButton( void );
void sysDraw_seekBar( uint8_t seekPos );
void sysDraw_cls( void );
uint8_t sysDraw_changeAlbumBG( char *path );
void sysDraw_ProgressBar1( uint16_t value ,uint16_t max );
void sysDraw_ProgressBar2( uint16_t value ,uint16_t max );

/* menu...c		*/
void menu001( void );
void menu002( void );
void menu010( void );
uint8_t menu011( void );
uint8_t menu012( void );
uint8_t menu013( void );
uint8_t menu014( void );
uint8_t menu015( void );
uint8_t menu016( void );
uint8_t menu017( void );
uint8_t menu018( void );
uint8_t menu019( void );
uint8_t menu020( void );
uint8_t menu021( void );
uint8_t menu022( void );
void menu030( void );

#endif
